/* 
    ssft - is a simple file transfer program that use SSL (Secure 
    Socket Layer) encryption to secure the transfers - and 
    signed certificates to authenticate users and servers. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Copyright 2001 by Jarle (jgaa) Aase, jgaa@jgaa.com
    http://www.jgaa.com

*/

#include "StdAfx.h"
#include "ssft.h"
#include <assert.h>

using namespace std;

char *get_time()
{
    static char buf[64];
    time_t now = time(NULL);
    struct tm *ptm = localtime(&now);
    if (!ptm)
        return "***notime***";
    
    strftime(buf, sizeof(buf), "%Y-%m-%d %H:%M", ptm);
    
    return buf;
}

int set_async_mode(SOCKET sck, bool makeAsync)
{    
    
#if WIN32
    unsigned long sck_mode = makeAsync;
    if (::ioctlsocket(sck, FIONBIO, &sck_mode))
        return -1;
#else
    int flags = 0;
    
    if ((flags = ::fcntl(sck, F_GETFL, 0)) != 0)
        return -1;
    
    if (makeAsync)
        flags |= O_NONBLOCK;
    else
        flags &= ~O_NONBLOCK;
    
    if (fcntl(sck, F_SETFL, flags) != 0)
        return -1;
#endif

    return 0;
}

void sleep(int seconds)
{
#ifdef WIN32
    Sleep(seconds * 1000);
#else
    alarm(seconds);
    pause();
#endif
}

SOCKET create_socket(bool doBind)
{
    SOCKET sck = socket(AF_INET, SOCK_STREAM, 0);
    if (sck == -1)
    {
        log << LOG_DATE << "Failed to create socket. " << GET_SOCKET_ERROR << ENDL;
        return -1;
    }
    
    pin->sin_port = htons(my_port);
    pin->sin_addr.s_addr = my_host;
    pin->sin_family = AF_INET;
    
    if (doBind)
    {
        if (bind(sck, &socaddr, sockaddr_len) != 0)
        {
            log << LOG_DATE << "Failed to bind socket. " << EXPLAIN_SOCKET_ERROR << ENDL;
            return -1;
        }
    }
    
    return sck;
}

bool resolve_hostname()
{
    my_host = inet_addr(hostname.c_str());
    if (((SOCKET)INADDR_NONE) == my_host)
    {
        if (verbose_flag)
        {
            log << LOG_DATE << "Looking up hostname " << hostname << " ... ";
            cout.flush();
        }
        
        struct hostent *pent = gethostbyname(hostname.c_str());
        
        if (NULL == pent)
        {
            if (verbose_flag)
                log << "FAILED" << ENDL;
            
            log << LOG_DATE << optarg << ": Unknown my_host" << ENDL;
            return false;
        }
        
        my_host = *(long *)pent->h_addr;
        
        if (verbose_flag)
        {
            in_addr my_addr;
            my_addr.s_addr = my_host;
            log << inet_ntoa(my_addr) << ENDL;
        }
    }

    return true;
}

bool is_dir(const char *path)
{
    struct stat st;
    string my_path = path;

    if (my_path.empty())
        return false;

    if (IS_SLASH(my_path[my_path.size() -1]))
        my_path.resize(my_path.size() -1);

    if (stat(my_path.c_str(), &st))
        return false;

    if ((st.st_mode & S_IFDIR) != S_IFDIR)
        return false;

    return true;
}

bool is_dir(const std::string& path)
{
    return is_dir(path.c_str());
}

bool is_file(const char *path)
{
    struct stat st;

    if (stat(path, &st))
        return false;

    if ((st.st_mode & S_IFREG) != S_IFREG)
        return false;

    return true;
}

bool is_file(const std::string& path)
{
    return is_file(path.c_str());
}

std::string resolve_path(const std::string& name)
{
    if (cert_dir.empty())
        return name;

    string my_name = name;
    const char *path_str = my_name.c_str();

#ifdef WIN32
    int i;
    while((i = my_name.find('\\')) != my_name.npos)
        my_name[i] = '/';
#endif

    if (path_str && *path_str && ('/' == *path_str))
        return name;

#ifdef WIN32
    if (*path_str)
    {
        if (isalpha(*path_str)
            && (':' == *++path_str))
            return name;
    }
#endif

    string path = cert_dir;

    if (!path.empty() && ('/' != path[path.size() -1]))
        path += '/';

    //cout << "cert_dir=" << cert_dir << ", name=" << name << endl;

    path += name;

#ifdef WIN32
    while((i = my_name.find('\\')) != my_name.npos)
        my_name[i] = '/';
#endif

    return path;
}

ssft_flen_t htonll(ssft_flen_t v)
{
    return 
#ifdef WIN32
     ((((v) & 0xff00000000000000) >> 56) 
      | (((v) & 0x00ff000000000000) >> 40)
      | (((v) & 0x0000ff0000000000) >> 24)
      | (((v) & 0x000000ff00000000) >> 8)
      | (((v) & 0x00000000ff000000) << 8)
      | (((v) & 0x0000000000ff0000) << 24)
      | (((v) & 0x000000000000ff00) << 40)
      | (((v) & 0x00000000000000ff) << 56));
#else
     ((((v) & 0xff00000000000000ull) >> 56) 
      | (((v) & 0x00ff000000000000ull) >> 40)
      | (((v) & 0x0000ff0000000000ull) >> 24)
      | (((v) & 0x000000ff00000000ull) >> 8)
      | (((v) & 0x00000000ff000000ull) << 8)
      | (((v) & 0x0000000000ff0000ull) << 24)
      | (((v) & 0x000000000000ff00ull) << 40)
      | (((v) & 0x00000000000000ffull) << 56));
#endif
}

std::string merge_paths(const std::string& v1,
                        const std::string& v2)
{
    string new_path = with_slash(v1);
    const char *p = v2.c_str();
    if (p)
    {
        if (IS_SLASH(*p))
            ++p;

        if (*p)
            new_path += p;
    }
    
    return new_path;
}

std::string with_slash(const std::string& v)
{
    if (v.empty())
    {
        static const char sys_slash[2] = {SYSSLASH, '\0'};
        return sys_slash;
    }
    else
    {
        char ch = v[v.size() -1];
        
#ifdef WIN32 
        if ('/' == ch)
        {
            string rval = v;
            rval[rval.size() -1] = SYSSLASH;
            return rval;
        }
#endif // WIN32 

        if (ch != SYSSLASH)
        {
            string rval = v;
            rval += SYSSLASH;
            return rval;
        }
    }

    return v;
}

void make_dirs(const std::string& path)
{
    string my_path;
    const char *p = path.c_str();
    bool virgin = true;

    if (!p)
        return;

    while(*p)
    {
        if (IS_SLASH(*p))
        {
            if (!virgin && !my_path.empty() && !is_dir(my_path))
            {
                if (mkdir(my_path.c_str() MKDIR_MODE) != 0)
                {
                    log << LOG_DATE
                        << "Warning: Failed to create directory \""
                        << my_path
                        << "\""
                        << ENDL;

                    return;
                }
            }

            my_path += SYSSLASH;
            p++;
            virgin = false;
        }
        else
        {
            my_path += *p++;
        }
    }
}

std::string parse_path(const std::string& path)
{
    typedef char path_char_t;
    
    std::vector<path_char_t> buffer(path.size() + 1);
    const path_char_t *p = path.c_str();
    path_char_t *pp = &buffer[0];
    int cnt = 0;
    
    while (*p)
    {
        if (IS_SLASH(*p))
        {
            // /C/.. /C/./any  /C/any/. /C/any/../.any /C/any\..
            if (p[1] == '.')
            {
                if (!p[2] || IS_SLASH(p[2]))
                {
                    // /C/any/. style. Just ignore
                    p += 2;
                    continue;
                }
                
                if (p[2] == '.')
                {
                    if (!p[3] || IS_SLASH(p[3]))
                    {
                        // /C/any/.. /C/any/../test style. Move back one
                        if (cnt == 0)
                            throw exception();
                        
                        while (--cnt && !IS_SLASH(pp[cnt]))
                        {
                            assert(cnt >= 0);
                        }
                        p += 3;
                        continue;
                    }
                    else
                    {
                        const path_char_t *ppp = p;
                        while(*ppp == '.')
                            ++ppp;
                        
                        // Deny /......[/]
                        if (!*ppp 
                            || IS_SLASH(*ppp) 
                            // We deny some common characters with special meanings as well!
                            // /..*../
                            || (*ppp == '*') 
                            || (*ppp == '?')
                            || (*ppp == '~'))
                        {
                            // Possible exploit
                            throw exception();
                        }
                    }
                }
            }
        }
        // Default
        pp[cnt++] = *p++;
    }
    
    pp[cnt++] = 0;
    
    return (const char *)&buffer[0];
}

